import os
import pygame
import sys

pygame.init()
size = width, height = 900, 700
screen = pygame.display.set_mode(size)
pygame.display.set_caption('The Roar')
blackness = pygame.Surface(size)
blackness.fill((0, 0, 0))
font = pygame.font.Font('data/LinLibertine_Re-4.7.5.ttf', 24)
clock = pygame.time.Clock()
pygame.event.set_allowed(None)
pygame.event.set_allowed([pygame.QUIT, pygame.KEYDOWN, pygame.MOUSEBUTTONUP])
def handle_event(event):
	if event.type == pygame.QUIT or event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
		file('save_game', 'w').write(repr((scene.__class__.__name__, [(s.__class__.__name__, s.visited) for s in all_scenes.values()], actions_performed)))
		sys.exit()
	if event.type == pygame.MOUSEBUTTONUP:
#		print event
		x, y = event.pos
		scene.click(x, y)

def load_image(name):
	return pygame.image.load(os.path.join('data', name))
def render(text, scene=None, shadow=2):
	if scene is None:
		scene = globals()['scene']
	surface = font.render(text, True, scene.text_color)
	if shadow:
		r, g, b = scene.text_color
		shadow_surface = font.render(text, True, (255 - r, 255 - g, 255 - b))
		w, h = surface.get_size()
		shadow_surface = pygame.transform.smoothscale(shadow_surface, (w / shadow, h / shadow))
		shadow_surface = pygame.transform.smoothscale(shadow_surface, (w, h))
		shadow_surface.blit(surface, (0, 0))
		surface = shadow_surface
	return surface

class Action(object):
	green_gem = load_image('green-gem.png')
	red_gem = load_image('red-gem.png')
	def __init__(self, x, y, **kwargs):
		assert len(kwargs) == 1
		self.name = kwargs.keys()[0]
		self.text = render(kwargs.values()[0])
		self.cancel = render('cancel')
		self.expanded = False
		self.fade = 0
		self.x = x - Action.green_gem.get_width() / 2
		self.y = y - Action.green_gem.get_height() / 2
	def expand(self, expand=True):
		for a in scene.actions:
			a.expanded = False
		self.expanded = expand
	def skip_animation(self, index):
		self.x = 700 - 60 * index
		self.y = 0
		if self.expanded:
			self.y = 20
	def draw(self, index):
		tx = 700 - 60 * index
		if self.x < tx:
			self.x += 10
			if self.x > tx:
				self.x = tx
		elif self.x > tx:
			self.x -= 10
			if self.x < tx:
				self.x = tx
		ty = 0
		if self.expanded:
			ty = 20
		if self.y < ty:
			self.y += 8
			if self.y > ty:
				self.y = ty
		elif self.y > ty:
			self.y -= 8
			if self.y < ty:
				self.y = ty
		if self.expanded:
			screen.blit(self.text, (width - self.text.get_rect().width - 50, 160))
			screen.blit(Action.red_gem, (tx, 150))
			x, y = pygame.mouse.get_pos()
			if tx + 90 < x < tx + 120 and 200 < y < 290:
				screen.blit(self.cancel, (tx + 105 - self.cancel.get_rect().width / 2, 290))
		screen.blit(Action.green_gem, (self.x, self.y))
	def click(self, scene, x, y):
		if self.x + 90 < x < self.x + 120 and y < 140:
			self.expand(not self.expanded)
			return True
		elif self.x + 90 < x < self.x + 120 and 200 < y < 290:
			actions_performed.remove((scene.caption, self.name))
			return True
		return False

class Scene(object):
	timeline = load_image('timeline.png')
	inventory_background = load_image('inventory.png')
	pin_image = load_image('time-pin.png')
	pin_font = pygame.font.Font('data/LinBiolinum_Bd-0.5.5.ttf', 8)
	visible_scenes = []
	def __init__(self):
		self.set_background(self.background)
		self.actions = []
		self.inventory = []
		self.item = None
		self.set_status(*self.status)
		self.set_caption(self.caption)
		self.visited = False
		self.past_cache = {}
		self.action_cache = None
	def set_caption(self, caption):
		self.caption = caption
		self.pin_caption = Scene.pin_font.render(caption, True, (0, 0, 0))
	def set_background(self, background):
		self.image = load_image(background)
	def set_status(self, *status):
		self.text = [render(s, scene=self) for s in status]
	def draw(self):
		screen.blit(self.image, (0, 50))
		for i, t in enumerate(self.text):
			screen.blit(t, (20, 60 + i * 60))
		for i, a in enumerate(self.actions):
			a.draw(i)
		screen.blit(Scene.inventory_background, (0, 0))
		ix = width / 2 - 25 * len(self.owned)
		for i, item in enumerate(self.owned):
			if self.item is not item:
				screen.blit(item.image, (ix + 50 * i, 0))
		if self.item:
			mx, my = pygame.mouse.get_pos()
			rect = self.item.image.get_rect()
			screen.blit(self.item.image, (mx - rect.width / 2, my - rect.height / 2))
		if blackness.get_alpha():
			screen.blit(blackness, (0, 0))
		screen.blit(Scene.timeline, (0, height - 50))
		for scene in self.visible_scenes:
			scene.draw_pin()
	def draw_pin(self):
		y = height - 140
		if scene is self:
			y -= 5
		screen.blit(Scene.pin_image, (self.x, y))
		screen.blit(self.pin_caption, (self.x + 98 - self.pin_caption.get_width() / 2, y + 80))
	def add_action(self, x, y, **kwargs):
		action = Action(x, y, **kwargs)
		self.actions.append(action)
		action.expand()
		return action
	def get_actions(self):
		if self.action_cache is None:
			allowed = set(name for (name, (x, y, text, scenes, kwargs)) in all_actions.items()
			              if isinstance(self, scenes))
			self.action_cache = [a for (t, a) in actions_performed if t == self.caption and a in allowed]
		return self.action_cache
	def has_action(self, name):
		return name in self.get_actions()
	def click(self, x, y):
		if 50 < y <= height - 100:  # Scene view.
			if any(a.click(self, x, y) for a in self.actions):
				self.sync()
				return
			points = self.get_points()
			if points:
				d2, p = min(((x - px) ** 2 + (y - py) ** 2, p) for (px, py, p) in points)
				if self.item:
					self.item.use(p)
				else:
					perform(self, p)
			self.sync()
		elif y > height - 100:  # Timeline.
			d, s = min((abs(x - s.x - 100), s) for s in Scene.visible_scenes)
			if s != self:
				new_scene(s)
		elif y < 50:  # Inventory.
			if self.item:
				self.item = None
			else:
				owned = [i for i in all_items_sorted if i.owned()]
				ix = width / 2 - 25 * len(owned)
				for i, item in enumerate(owned):
					if x - 50 < ix + 50 * i < x:
						self.item = item
	def flush(self):
		self.past_cache = {}
		self.action_cache = None
	def sync(self):
		for s in all_scenes.values():
			s.flush()
		self.sync_actions()
		self.sync_visible_scenes()
		self.sync_inventory()
	def skip_animation(self):
		for i, a in enumerate(self.actions):
			a.skip_animation(i)
	def sync_visible_scenes(self):
		Scene.visible_scenes = visible_scenes()
	def sync_inventory(self):
		self.owned = [i for i in all_items_sorted if i.owned()]
	def sync_actions(self):
		true_actions = set(self.get_actions())
		for a in self.actions[:]:
			if a.name not in true_actions:
				self.actions.remove(a)
		displayed_actions = set(a.name for a in self.actions)
		for a in self.get_actions():
			if a not in displayed_actions:
				x, y, text, scenes, kwargs = all_actions[a]
				self.add_action(x, y, **{a: text})
	def prior_scenes(self):
		return [s for s in all_scenes.values() if s.x < self.x and s.visible()]
	def has_happened(self, action, recently=False):
		if (action, recently) not in self.past_cache:
			for scene in self.prior_scenes():
				if scene.has_action(action):
					result = True
					break
			else:
				result = recently and self.has_action(action)
			self.past_cache[(action, recently)] = result
		return self.past_cache[(action, recently)]
	def internal_visited(self):
		self.item = None
		self.visited = True
		self.adjust()
	def adjust(self):
		pass
	def live(self):
		return self.visited and self.visible()
	def get_points(self):
		return [(x, y, name) for (name, (x, y, text, scenes, kwargs)) in all_actions.items()
		        if isinstance(self, scenes) and not kwargs.get('item')]

def visible_scenes():
	return sorted([s for s in all_scenes.values() if s.visible()], key=lambda s: s.x)
def live(scene_class):
	return all_scenes[scene_class.__name__].live()

class HomeMorning(Scene):
	background = 'home.png'
	status = 'You crawl out of the bed.',
	text_color = 255, 255, 255
	x = 60
	caption = '8:00'
	def visible(self):
		return True

all_actions = {}
def define_action(name, x, y, text, *scenes, **kwargs):
	assert name not in all_actions
	all_actions[name] = x, y, text, scenes, kwargs

define_action('go_to_work', 844, 428, 'You take off to work.', HomeMorning)
define_action('put_on_glasses', 733, 408, 'You put on your glasses.', HomeMorning)
define_action('take_keys', 701, 408, 'You take your keys.', HomeMorning)
define_action('make_bed', 518, 337, 'You spend some time tidying up the bed.', HomeMorning)

class StreetWalk(Scene):
	background = 'street.png'
	status = 'You are walking to work.',
	text_color = 0, 0, 0
	x = 100
	caption = '9:00'
	def visible(self):
		return (live(HomeMorning) and
		        self.has_happened('go_to_work') and
		        self.has_happened('make_bed') and
		        self.has_happened('put_on_glasses'))
class StreetBus(Scene):
	background = 'street-with-bus.png'
	status = 'You are walking to work.',
	text_color = 0, 0, 0
	x = 100
	caption = '9:00'
	def visible(self):
		return (live(HomeMorning) and
		        self.has_happened('go_to_work') and
		        not self.has_happened('make_bed') and
		        self.has_happened('put_on_glasses'))

define_action('drink_coffee', 191, 390, 'You have a refreshing coffee.', StreetWalk, StreetBus)
define_action('buy_flowers', 367, 398, 'You buy a bouquet of daffodils.', StreetWalk, StreetBus)
define_action('take_bus', 733, 518, 'You take the bus to work.', StreetBus)

class StreetDead(Scene):
	background = 'street-dead.png'
	status = 'You were run over by a car.', 'Go back to the previous scene and take precautions!'
	text_color = 0, 0, 0
	x = 100
	caption = '9:00'
	def visible(self):
		return (live(HomeMorning) and
		        self.has_happened('go_to_work') and
		        not self.has_happened('put_on_glasses'))

class Circus1(Scene):
	background = 'circus.png'
	status = 'You arrive at work.',
	text_color = 0, 0, 0
	x = 140
	caption = '10:00'
	def visible(self):
		return self.has_happened('take_bus')

class Circus2(Scene):
	background = 'circus.png'
	status = 'You are hanging out with the magician.',
	text_color = 0, 0, 0
	x = 180
	caption = '11:00'
	def visible(self):
		return ((live(StreetWalk) or live(StreetBus)) and
		        not self.has_happened('go_to_lion') and not self.has_happened('distract_acrobat'))
	def adjust(self):
		if all_scenes['Circus1'].live():
			self.set_status(*Circus2.status)
		else:
			self.set_status(*Circus1.status)

class Circus2WithoutAcrobat(Scene):
	background = 'circus-no-girl.png'
	status = 'You are hanging out with the magician.',
	text_color = 0, 0, 0
	x = 180
	caption = '11:00'
	def visible(self):
		return self.has_happened('distract_acrobat') and not self.has_happened('go_to_lion')

define_action('talk_to_magician', 583, 436, 'You tell the magician about the trap.', Circus1, Circus2, Circus2WithoutAcrobat)
define_action('greet_acrobat', 722, 125, 'You wave at the pretty acrobat girl.', Circus1, Circus2)
define_action('go_to_lion', 77, 455, 'You make your way to the lion\'s cage.', Circus1, Circus2, Circus2WithoutAcrobat, exclusive='circus')
define_action('go_to_trapeze', 722, 125, 'You climb up to the trapeze.', Circus2WithoutAcrobat, exclusive='circus')

class Lion(Scene):
	background = 'cage-girl-meat.png'
	status = 'The lion stares back at you with hunger in her eyes.',
	text_color = 0, 0, 0
	x = 180
	caption = '11:00'
	def visible(self):
		return self.has_happened('go_to_lion') and not self.has_happened('distract_acrobat')
class LionWithoutAcrobat(Scene):
	background = 'cage-no-girl-meat.png'
	status = 'The lion stares back at you with hunger in her eyes.',
	text_color = 0, 0, 0
	x = 180
	caption = '11:00'
	def visible(self):
		return self.has_happened('go_to_lion') and self.has_happened('distract_acrobat')
define_action('take_meat', 80, 424, 'You take the juicy steak.', Lion, LionWithoutAcrobat)
define_action('pet_lion', 309, 333, 'You stare at the lion.', Lion, LionWithoutAcrobat)
define_action('go_to_grounds', 697, 330, 'You make your way back to the grounds.', Lion, LionWithoutAcrobat, exclusive='lion')
define_action('greet_acrobat_from_lion', 737, 107, 'You wave at the trapeze girl.', Lion)
define_action('go_to_trapeze_from_lion', 737, 107, 'You climb up to the trapeze.', LionWithoutAcrobat, exclusive='lion')

class Trapeze(Scene):
	background = 'trapeze.png'
	status = 'You look down into the dizzying depth and observe.', 'The magician opened a portal but was captured him before he could enter it.'
	text_color = 0, 0, 0
	x = 300
	caption = '14:00'
	def visible(self):
		return self.has_happened('go_to_trapeze') or self.has_happened('go_to_trapeze_from_lion')
define_action('leave_earth', 309, 333, 'You dive into the dimensional portal.', Trapeze)

class Circus4(Scene):
	background = 'circus-no-magician-no-girl.png'
	status = 'You are alone in the circus.',
	text_color = 0, 0, 0
	x = 340
	caption = '15:00'
	def visible(self):
		return live(Trapeze) and not self.has_happened('leave_earth')

class Alienia(Scene):
	background = 'alienia.png'
	status = 'You emerge in a strange place.',
	text_color = 0, 0, 0
	x = 340
	caption = '?'
	def visible(self):
		return self.has_happened('leave_earth')
define_action('poke_monster', 265, 482, 'You stick your head in the monster\'s mouth!', Alienia)
define_action('take_ray', 529, 251, 'You wrestle the ray into your pocket.', Alienia)
define_action('return_to_earth', 162, 244, 'You dive into the dimensional portal.', Alienia)

class SWAT(Scene):
	background = 'circus-swat.png'
	status = 'A SWAT team has arrived.', 'They ejected you from the circus grounds.', 'You wait until they leave.'
	text_color = 0, 0, 0
	x = 300
	caption = '14:00'
	def visible(self):
		return ((live(Circus2) or live(Circus2WithoutAcrobat) or live(Lion) or live(LionWithoutAcrobat)) and
		        not self.has_happened('go_to_trapeze') and not self.has_happened('go_to_trapeze_from_lion'))

class Show(Scene):
	background = 'show.png'
	status = 'The time of your performance has come!',
	text_color = 255, 255, 255
	x = 460
	caption = '18:00'
	def visible(self):
		return live(SWAT) or live(Circus4)
class ShowPortal(Scene):
	background = 'show.png'
	status = 'You jump out of the magician\'s hat!', 'The crowd goes wild!'
	text_color = 255, 255, 255
	x = 460
	caption = '18:00'
	def visible(self):
		return self.has_happened('feed_monster') and self.has_happened('return_to_earth')
define_action('stick_head_in', 367, 398, 'You stick your head in the lion\'s mouth!', Show, ShowPortal, exclusive='show')
define_action('jump_in_hat', 138, 487, 'You dive into the magician\'s hat!', Show, ShowPortal, exclusive='show')
def head_bit_off(scene):
	return scene.has_happened('stick_head_in') and not scene.has_happened('feed_lion')

class AlieniaLater(Scene):
	background = 'alienia-closed.png'
	status = 'You emerge in a strange place.',
	text_color = 0, 0, 0
	x = 540
	caption = '?'
	def visible(self):
		return self.has_happened('jump_in_hat')
define_action('poke_monster_later', 265, 482, 'You stick your head in the monster\'s mouth!', AlieniaLater)
define_action('eat_small_mushroom', 521, 456, 'Eat the tiny mushroom.', Alienia, AlieniaLater)
define_action('eat_big_mushroom_1', 314, 156, 'You take a bite from the giant mushroom.', Alienia, AlieniaLater)
define_action('eat_big_mushroom_2', 855, 285, 'You take a bite from the giant mushroom.', Alienia, AlieniaLater)
define_action('look_at_portal', 162, 244, 'The portal closes up before you get there.', AlieniaLater)

class HomeEveningSleepy(Scene):
	background = 'home.png'
	status = 'You get home and immediately fall asleep.',
	text_color = 255, 255, 255
	x = 620
	caption = '22:00'
	def visible(self):
		return (live(Show) or live(ShowPortal)) and not head_bit_off(self) and not self.has_happened('drink_coffee') and not self.has_happened('jump_in_hat')
	
class HomeEvening(Scene):
	background = 'home.png'
	status = 'You get home.',
	text_color = 255, 255, 255
	x = 620
	caption = '22:00'
	def visible(self):
		return (live(Show) or live(ShowPortal)) and not head_bit_off(self) and self.has_happened('drink_coffee') and not self.has_happened('jump_in_hat')
define_action('lock_door', 844, 428, 'You lock your door.', HomeEvening)
define_action('look_at_glasses', 733, 408, 'You clean your glasses.', HomeEvening)
define_action('look_at_keys', 701, 408, 'You play with your keys.', HomeEvening)
define_action('get_in_bed', 518, 337, 'You get in bed.', HomeEvening)
define_action('look_at_window', 241, 220, 'You look out the window and think about your secret plans.', HomeMorning, HomeEvening)
define_action('look_at_floor', 241, 500, 'You sweep the floor.', HomeMorning, HomeEvening)
def trap_set(scene):
	return scene.has_happened('place_ray') and scene.has_happened('place_lion')

class EpilogueDayOff(Scene):
	background = 'home.png'
	status = 'You missed work and were thrown out of the company.', 'If you want a better ending, go back to 8:00 and click the door!',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return not self.has_happened('go_to_work')
class EpilogueRanOver(Scene):
	background = 'street-dead.png'
	status = 'A car ran you over. You died.', 'Go back to the first scene and take precautions!',
	text_color = 0, 0, 0
	x = 700
	caption = ''
	def visible(self):
		return self.has_happened('go_to_work') and not self.has_happened('put_on_glasses')
class EpilogueNoShow(Scene):
	background = 'home.png'
	status = 'You did not perform your show.', 'The audience and your boss were disappointed.',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return (live(Show) or live(ShowPortal)) and not self.has_happened('stick_head_in') and not self.has_happened('shrink_lion') and not self.has_happened('jump_in_hat')
class EpilogueHeadBitOff(Scene):
	background = 'home.png'
	status = 'A hungry lion bit off your head.', 'The audience was entertained.',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return head_bit_off(self)
class EpilogueShowDone(Scene):
	background = 'home.png'
	status = 'You thrilled the audience with a great show!', 'However a SWAT team crashed through your window and', 'adbucted you to a research facility in the middle of the night.',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return self.has_happened('stick_head_in') and not head_bit_off(self) and not self.has_happened('jump_in_hat')
class EpilogueAlienia(Scene):
	background = 'alienia-closed.png'
	status = 'You live out your life in a grotesque world under magenta skies.',
	text_color = 0, 0, 0
	x = 700
	caption = ''
	def visible(self):
		return (self.has_happened('leave_earth') and not self.has_happened('return_to_earth') or self.has_happened('jump_in_hat')) and not self.has_happened('poke_monster') and not self.has_happened('poke_monster_later')
class EpilogueMonsterDeath(Scene):
	background = 'alienia-closed.png'
	status = 'As you were jumping into the portal the alien monster bit you in two.',
	text_color = 0, 0, 0
	x = 700
	caption = ''
	def visible(self):
		return self.has_happened('return_to_earth') and not self.has_happened('feed_monster') and not self.has_happened('poke_monster') and not self.has_happened('shrink_monster')
class EpilogueMiniMonsterDeath(Scene):
	background = 'alienia-closed.png'
	status = 'As you were jumping into the portal the alien monster grabbed your leg and nibbled you to death.',
	text_color = 0, 0, 0
	x = 700
	caption = ''
	def visible(self):
		return self.has_happened('return_to_earth') and not self.has_happened('feed_monster') and not self.has_happened('poke_monster') and self.has_happened('shrink_monster')
class EpilogueMonsterPokeDeath(Scene):
	background = 'alienia-closed.png'
	status = 'Perhaps not unexpectedly the scary alien monster bit off your head.',
	text_color = 0, 0, 0
	x = 700
	caption = ''
	def visible(self):
		return self.has_happened('poke_monster') or self.has_happened('poke_monster_later')
class EpilogueShowPortal(Scene):
	background = 'home.png'
	status = 'You gave the audience a show of a lifetime!', 'However a SWAT team crashed through your window in the middle', 'of the night and adbucted you to a research facility.',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return self.has_happened('shrink_lion') and not self.has_happened('place_ray') and not self.has_happened('place_lion')
class EpilogueShowPortal(Scene):
	background = 'home.png'
	status = 'You gave the audience a show of a lifetime!', 'However a SWAT team crashed through your window in the middle', 'of the night and adbucted you to a research facility.',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return self.has_happened('shrink_lion') and not self.has_happened('place_ray') and not self.has_happened('place_lion')
class EpilogueShrinkNoLion(Scene):
	background = 'home.png'
	status = 'You gave the audience a show of a lifetime!', 'However a miniaturized SWAT team adbucted you to a research facility', 'in the middle of the night.',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return self.has_happened('place_ray') and not self.has_happened('place_lion')
class EpilogueLionNoShrink(Scene):
	background = 'home.png'
	status = 'You gave the audience a show of a lifetime!', 'However a SWAT team crashed through your window, stomped over your', 'miniature lion and adbucted you to a research facility.',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return self.has_happened('place_lion') and not self.has_happened('place_ray')
class EpilogueWin(Scene):
	background = 'home.png'
	status = 'You gave the audience a show of a lifetime.', 'Then you fed an entire SWAT team to your mini-lion.', 'And later you went on to take over Earth!', 'Congratulations!',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return trap_set(self) and (not self.has_happened('talk_to_magician') or self.has_happened('lock_door'))
class EpilogueSWATThroughDoor(Scene):
	background = 'home.png'
	status = 'You gave the audience a show of a lifetime!', 'However a SWAT team walked through your door in the middle', 'of the night and adbucted you to a research facility.',
	text_color = 255, 255, 255
	x = 700
	caption = ''
	def visible(self):
		return trap_set(self) and self.has_happened('talk_to_magician') and not self.has_happened('lock_door')

class Item(object):
	def __init__(self):
		self.image = load_image(self.icon)
	def use(self, point):
		for name, (p, item) in all_item_actions.items():
			if p == point and isinstance(self, item):
				perform(scene, name)
				scene.item = None

all_item_actions = {}
def define_item_action(name, point, text, item, **kwargs):
	assert name not in all_actions
	kwargs['item'] = True
	x, y, t, scenes, kw = all_actions[point]
	all_actions[name] = x, y, text, all_actions[point][3], kwargs
	all_item_actions[name] = point, item

class Bouquet(Item):
	icon = 'bouquet.png'
	def owned(self):
		return scene.has_happened('buy_flowers', recently=True) and not scene.has_happened('distract_acrobat', recently=True) and not scene.has_happened('feed_lion_flower', recently=True) and not scene.has_happened('feed_lion_flower_show', recently=True) and not scene.has_happened('give_magician_flower', recently=True)
define_item_action('distract_acrobat', 'greet_acrobat', 'You dazzle the acrobat with flowers. She wanders off in search of a vase.', Bouquet)
define_item_action('distract_acrobat_from_lion', 'greet_acrobat_from_lion', 'You dazzle the acrobat with flowers. She wanders off in search of a vase.', Bouquet)
define_item_action('feed_lion_flower', 'pet_lion', 'You dazzle the lion with flowers.', Bouquet)
define_item_action('feed_lion_flower_show', 'stick_head_in', 'You taunt the lion with flowers.', Bouquet)
define_item_action('give_magician_flower', 'talk_to_magician', 'The magician vanishes your bouquet.', Bouquet)

class Meat(Item):
	icon = 'meat.png'
	def owned(self):
		return scene.has_happened('take_meat', recently=True) and not scene.has_happened('feed_lion', recently=True) and not scene.has_happened('feed_monster', recently=True)
define_item_action('feed_lion', 'pet_lion', 'You feed the lion a nice steak.', Meat)
define_item_action('feed_monster', 'poke_monster', 'You feed the monster a nice steak.', Meat)

class MiniLion(Item):
	icon = 'lion.png'
	def owned(self):
		return scene.has_happened('shrink_lion', recently=True) and not scene.has_happened('place_lion', recently=True)
define_item_action('place_lion', 'look_at_floor', 'You let the miniature lion loose in the apartment.', MiniLion)

class ShrinkRay(Item):
	icon = 'ray.png'
	def owned(self):
		return scene.has_happened('take_ray', recently=True) and not scene.has_happened('place_ray', recently=True)
define_item_action('place_ray', 'look_at_window', 'You place the shrink ray above the window.', ShrinkRay)
define_item_action('shrink_lion', 'stick_head_in', 'The ray shrinks the lion! The crowd is ecstatic!', ShrinkRay)
define_item_action('shrink_monster', 'poke_monster', 'The monster shrinks a bit.', ShrinkRay)

def my_issubclass(c, d):
	if c is d:
		return False
	try:
		return issubclass(c, d)
	except TypeError:
		return False
def get_items():
	return dict((k, v()) for (k, v) in globals().items() if my_issubclass(v, Item))
all_items = get_items()
all_items_sorted = [v for (k, v) in sorted(all_items.items())]
def get_scenes():
	return dict((k, v()) for (k, v) in globals().items() if my_issubclass(v, Scene))
all_scenes = get_scenes()

actions_performed = []
def perform(scene, action):
	x, y, t, scenes, kwargs = all_actions[action]
	exclusive = kwargs.get('exclusive')
	if exclusive:
		for c, a in actions_performed[:]:
			x, y, t, scenes, kwargs = all_actions[a]
			if kwargs.get('exclusive') == exclusive:
				actions_performed.remove((c, a))
	actions_performed.append((scene.caption, action))

def scenes():
	while True:
		yield next_scene
def new_scene(scene):
	global next_scene, flip_scene
	next_scene = scene
	flip_scene = True
next_scene = all_scenes['HomeMorning']
flip_scene = False

try:
	n, visited, actions_performed = eval(file('save_game').read())
	for k, v in visited:
		all_scenes[k].visited = v
	next_scene = all_scenes[n]
	print 'Save game loaded.'
except:
	pass

def main():
	global scene, flip_scene
	for scene in scenes():
		scene.internal_visited()
		for s in all_scenes.values():
			s.flush()
		scene.sync()
		scene.skip_animation()
		fade = 0
		while True:
			for event in pygame.event.get():
				handle_event(event)
			blackness.set_alpha(255 - fade)
			scene.draw()
			if flip_scene:
				if fade <= 0:
					flip_scene = False
					break
				fade -= 50
				if fade < 0:
					fade = 0;
			elif fade != 255:
				fade += 30
				if fade >= 255:
					fade = 255
			pygame.display.flip()
			clock.tick(40)
#			print clock.get_fps()

if __name__ == '__main__':
	main()